@@ -8,4 +8,7 @@
 #include "winrt/Composable.h"
 
 // This is used to validate WRL interop support.
+#pragma warning(push)
+#pragma warning(disable: 4324) // structure was padded due to alignment specifier
 #include <wrl.h>
+#pragma warning(pop)
\ No newline at end of file
